import 'package:dartdap/dartdap.dart';
import 'package:logging/logging.dart';

/// An example of performing a paged search using the SimplePagedResults Control
/// Tested against ForgeRock DS version 7.0
///
/*
dsconfig create-backend-vlv-index \
--backend-name amIdentityStore \
--set base-dn:ou=people,ou=identities \
--set filter:'(&)' \
--set scope:subordinate-subtree \
--set sort-order:'+uid' \
--type generic \
--index-name uid \
--hostname localhost \
--port 4444 \
--bindDn uid=admin \
--trustAll \
--bindPassword password \
--no-prompt

*/

main() async {

  Logger.root.onRecord.listen((LogRecord rec) {
    print('${rec.time}: ${rec.loggerName}: ${rec.level.name}: ${rec.message}');
  });

  Logger.root.level = Level.INFO;

  var ldap = LdapConnection(
      host: 'localhost', port: 1389, bindDN: 'uid=admin', password: 'password');

  var baseDN = 'ou=people,ou=identities';
  var attrs = ['dn', 'uid', 'sn'];
  //var vlv = VLVRequestControl.newVLVSearch();
  var vlv = VLVRequestControl.assertionControl("user.2000", 10, 10);
  var serverSideSortControl = ServerSideSortRequestControl([SortKey("uid")]);

  var results = await ldap.query(baseDN, '(uid=*)', attrs,
       controls: [serverSideSortControl,vlv], scope: SearchScope.ONE_LEVEL);

  await for (var entry in results.stream) {
    print("Got entry ${entry.dn} attrs = ${entry.attributes}");
  }

  // This works...
  results.controls.forEach((control) => print("Got control $control"));
//
//  var pc = results.controls[0] as SimplePagedResultsControl;
//  pc.size = 10;
//  // search again...
//  var r2 = await ldap.query(baseDN,  '(objectclass=*)', attrs, controls: [pc]);
//
//  print("Next result set  results = $r2");
//  await for (var entry in r2.stream) {
//    print("Got entry ${entry.dn} attrs = ${entry.attributes}");
//  }

  await ldap.close();

}

